package weka.classifiers.timeseries.eval;

import java.util.List;

import weka.classifiers.evaluation.NumericPrediction;
import weka.core.Instance;
import weka.core.Utils;

public class DACModule extends ErrorModule {

  protected Instance previousInstance;
  
  protected double[] m_correct;
  protected double[] m_directionsCount;
  
  public String getEvalName() {
    return "DAC";
  }
  
  public String getDescription() {
    return "Direction accuracy";
  }
  
  public String getDefinition() {
    return "count(sign(actual_current - actual_previous) == " +
    		"sign(pred_current - pred_previos)) / N";
  }
  
  public void evaluateForInstance(List<NumericPrediction> forecasts,
      Instance inst) throws Exception {
    super.evaluateForInstance(forecasts, inst);
    
    if (m_predictions.get(0).size() > 1) {
      for (int i = 0; i < m_targetFieldNames.size(); i++) {
        NumericPrediction currentForI = 
          m_predictions.get(i).get(m_predictions.get(i).size() - 1);
        NumericPrediction previousForI =
          m_predictions.get(i).get(m_predictions.get(i).size() - 2);
        
        if (!Utils.isMissingValue(currentForI.predicted()) &&
            !Utils.isMissingValue(previousForI.predicted()) &&
            !Utils.isMissingValue(currentForI.actual()) &&
            !Utils.isMissingValue(previousForI.actual())) {
          double predictedDirection = 
            currentForI.predicted() - previousForI.predicted();
          double actualDirection = 
            currentForI.actual() - previousForI.actual();
          
          if (actualDirection > 0 && predictedDirection > 0) {
            m_correct[i]++;            
          } else if (actualDirection < 0 && predictedDirection < 0) {
            m_correct[i]++;
          } else if (actualDirection == 0 && predictedDirection == 0) {
            m_correct[i]++;
          }
          
          m_directionsCount[i]++;
        }
      }
    } else {
      m_correct = new double[m_targetFieldNames.size()];
      m_directionsCount = new double[m_targetFieldNames.size()];
    }
  }
  
  public double[] calculateMeasure() throws Exception {
    double[] result = new double[m_targetFieldNames.size()];
    for (int i = 0; i < result.length; i++) {
      result[i] = Utils.missingValue();
    }
    
    for (int i = 0; i < m_targetFieldNames.size(); i++) {
      if (m_directionsCount[i] > 0) {
        result[i] = m_correct[i] / m_directionsCount[i] * 100.0;
      }
    }
    
    return result;
  }
}
