/**
 * 
 */
package weka.classifiers.timeseries.eval;

import java.util.List;

import weka.classifiers.evaluation.NumericPrediction;
import weka.core.Utils;

/**
 * @author mhall
 *
 */
public class MAPEModule extends ErrorModule {

  public String getEvalName() {
    return "MAPE";
  }
  
  public String getDescription() {
    return "Mean absolute percentage error";
  }
  
  public String getDefinition() {
    return "sum(abs((predicted - actual) / actual)) / N";
  }
  
  public double[] calculateMeasure() throws Exception {
    double[] result = new double[m_targetFieldNames.size()];
    for (int i = 0; i < result.length; i++) {
      result[i] = Utils.missingValue();
    }
    
    for (int i = 0; i < m_targetFieldNames.size(); i++) {
      double sumAbs = 0;
      List<NumericPrediction> preds = m_predictions.get(i);

      int count = 0;
      for (NumericPrediction p : preds) {
        if (!Utils.isMissingValue(p.error())) {
          sumAbs += Math.abs(p.error() / p.actual());
          count++;
        }
      }
      
      if (m_counts[i] > 0) {
        sumAbs /= m_counts[i];
      }
      
      if (count > 0) {
        result[i] = sumAbs * 100.0;
      } else {
        result[i] = Utils.missingValue();
      }
    }
    
    return result;
  }  
}
