package weka.classifiers.timeseries.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;

import weka.classifiers.Classifier;
import weka.classifiers.timeseries.TSLagMaker;
import weka.classifiers.timeseries.WekaForecaster;
import weka.classifiers.timeseries.eval.TSEvalModule;
import weka.classifiers.timeseries.eval.TSEvaluation;
import weka.core.Attribute;
import weka.core.Capabilities;
import weka.core.Capabilities.Capability;
import weka.core.Instances;
import weka.core.Range;
import weka.gui.AttributeSelectionPanel;
import weka.gui.GenericObjectEditor;
import weka.gui.PropertyPanel;

public class AdvancedConfigPanel extends JPanel {
  
  /** The training instances to operate on */
  protected Instances m_instances;
  
  /** A reference to the simple config panel */
  protected SimpleConfigPanel m_simpleConfig;
  
  JTabbedPane m_configHolder = new JTabbedPane();
  
  /** Editor for selecting and configuring the base algorithm */
  protected GenericObjectEditor m_baseLearnerEditor = 
    new GenericObjectEditor();
  protected PropertyPanel m_baseLearnerPanel = 
    new PropertyPanel(m_baseLearnerEditor);
  
  // lagging stuff
  protected JCheckBox m_useCustomLags = 
    new JCheckBox("Use custom lag lengths");
  
  protected JSpinner m_minLagSpinner;
  protected JSpinner m_maxLagSpinner;
  
  protected JCheckBox m_adjustForVarianceCheckBox = 
    new JCheckBox("Adjust for variance");
  
  protected JTextField m_fineTuneLagsField = new JTextField();
  
  protected JCheckBox m_averageLongLags =
    new JCheckBox("Average consecutive long lags");
  protected JSpinner m_averageLagsAfter;
  protected JSpinner m_numConsecutiveToAverage;
  
  // date-derived periodic stuff
  Instances m_dateDerivedPeriodicsHeader;
  protected JCheckBox m_customizeDateDerivedPeriodics = 
    new JCheckBox("Customize");
  protected AttributeSelectionPanel m_dateDerivedPeriodicSelector = 
    new AttributeSelectionPanel(false, false, false, false);
  
  // non date-based primary periodic stuff
  protected JComboBox m_primaryPeriodicCombo = new JComboBox();
  
  // Evaluation stuff
  protected Instances m_evaluationModsHeader;
  protected AttributeSelectionPanel m_evaluationMetrics = 
    new AttributeSelectionPanel(false, false, false, false);
  protected JCheckBox m_trainingCheckBox = new JCheckBox("Evaluate on training");
  protected JCheckBox m_holdoutCheckBox = new JCheckBox("Evaluate on held out training");
  protected JTextField m_holdoutSize = new JTextField();
  
  /** show the separate test set checkbox and button? */
  protected boolean m_allowSeparateTestSet = true;
  protected JCheckBox m_separateTestSetCheckBox = 
    new JCheckBox("Evaluate on a separate test set");
  protected JButton m_testSetBut = new JButton("Separate test set");
  
  // output stuff
  protected JCheckBox m_outputPredsCheckBox = 
    new JCheckBox("Output predictions at step");
  protected JComboBox m_outputPredsCombo = new JComboBox();
  protected JLabel m_outputPredsComboLabel = new JLabel("Target to output", JLabel.RIGHT);
  protected JSpinner m_outputStepSpinner;
  protected JLabel m_outputStepLabel = new JLabel("Step to output", JLabel.RIGHT);
  protected JCheckBox m_outputFutureCheckBox = 
    new JCheckBox("Output future predictions beyond end of series ");
  
  // graphing stuff
  protected JCheckBox m_graphPredsAtStepCheckBox =
    new JCheckBox("Graph predictions at step");
  protected JSpinner m_graphPredsAtStepSpinner;
  protected JLabel m_stepLab = new JLabel("Steps to graph");
  protected JCheckBox m_graphTargetForStepsCheckBox =
    new JCheckBox("Graph target at steps:");
  protected JLabel m_targetComboLabel = new JLabel("Target to graph", JLabel.RIGHT);
  protected JComboBox m_graphTargetAtStepsCombo = new JComboBox();
  protected JTextField m_stepRange = new JTextField("1");
  protected JCheckBox m_graphFutureCheckBox =
    new JCheckBox("Graph future predictions beyond end of series");
  
  static {
    GenericObjectEditor.registerEditors();
  }
  
  public String getTabTitle() {
    return "Advanced configuration";
  }
  
  public String getTabTitleToolTip() {
    return "Advanced configuration";
  }
  
  public Classifier getBaseClassifier() {
    return (Classifier)m_baseLearnerEditor.getValue();
  }
  
  public void setInstances(Instances train) {
    m_instances = train;
    updatePanel();
  }
  
  public void enableDateDerivedPeriodics(boolean s) {
    m_customizeDateDerivedPeriodics.setEnabled(s);
    if (!s) {
      m_customizeDateDerivedPeriodics.setSelected(false);
    }
  }
  
  public boolean isEnabledCustomizeDateDerivedPeriodics() {
    return m_customizeDateDerivedPeriodics.isEnabled();
  }
  
  public boolean getCustomizeDateDerivedPeriodics() {
    return m_customizeDateDerivedPeriodics.isSelected();
  }
  
  public boolean isUsingCustomLags() {
    return m_useCustomLags.isSelected();
  }
  
  public double getHoldoutSetSize() {
    double result = 0;
    
    if (m_holdoutCheckBox.isSelected()) {
      try {
        result = Double.parseDouble(m_holdoutSize.getText());
      } catch (NumberFormatException ex) {}
    }
    
    return result;
  }
  
  public boolean getOutputFuturePredictions() {
    return m_outputFutureCheckBox.isSelected();
  }
  
  /**
   * Returns at which step to output predictions. Returns
   * 0 if the user has not opted to output predictions.
   * 
   * @return step at which to output predictions or 0 if
   * no predictions are to be output.
   */
  public int getOutputPredictionsAtStep() {
    if (!m_outputPredsCheckBox.isSelected()) {
      return 0;
    }
    
    return ((SpinnerNumberModel)m_outputStepSpinner.
        getModel()).getNumber().intValue();
  }
  
  public boolean getGraphTargetForSteps() {
    return m_graphTargetForStepsCheckBox.isSelected();
  }
  
  public String getGraphTargetForStepsTarget() {
    if (!getGraphTargetForSteps()) {
      return null;
    }
    
    return m_graphTargetAtStepsCombo.getSelectedItem().toString();
  }
  
  public List<Integer> getGraphTargetForStepsStepList() {
    String rng = m_stepRange.getText();
    
    if (rng == null || rng.length() == 0) {
      return null;
    }
    
    Range range = new Range(rng);
    range.setUpper(m_simpleConfig.getHorizonValue());
    int[] indices = range.getSelection();
    
    List<Integer> rangeList = new ArrayList<Integer>();
    for (int i : indices) {
      rangeList.add((i + 1));
    }
    
    return rangeList;
  }
  
  /**
   * Get the selected target to output predictions for. Returns
   * null if the user has not opted to output predictions.
   * 
   * @return the name of the target to output predictions for or
   * null if no predictions are to be output.
   */
  public String getOutputPredictionsTarget() {
    if (!m_outputPredsCheckBox.isSelected()) {
      return null;
    }
    
    return m_outputPredsCombo.getSelectedItem().toString();
  }
  
  public int getGraphPredictionsAtStep() {
    if (!m_graphPredsAtStepCheckBox.isSelected()) {
      return 0;
    }
    
    return ((SpinnerNumberModel)m_graphPredsAtStepSpinner.
        getModel()).getNumber().intValue();
  }
  
  public boolean getGraphFuturePredictions() {
    return m_graphFutureCheckBox.isSelected();
  }
  
  public AdvancedConfigPanel(SimpleConfigPanel s, 
      boolean allowSeparateTestSet) {
    
    m_allowSeparateTestSet = allowSeparateTestSet;
    m_simpleConfig = s;

    setLayout(new BorderLayout());
    layoutLearnerPanel();
    layoutLagPanel();        
    layoutDateDerivedPeriodicPanel();
    layoutEvaluationPanel();
    layoutOutputPanel();

    add(m_configHolder, BorderLayout.CENTER);
  }
  
  public AdvancedConfigPanel(SimpleConfigPanel s) {
    this(s, true);
  }
  
  public void updatePanel() {
    updateDateDerivedPanel();
    updatePrimaryPeriodic();
    updateOutputPanel();
  }
  
  public void updateOutputPanel() {
    if (m_simpleConfig.m_targetHeader != null) {
      // configure the target combos
      Vector<String> candidates = new Vector<String>();
      for (int i = 0; i < m_simpleConfig.m_targetHeader.numAttributes(); i++) {
        candidates.add(m_simpleConfig.m_targetHeader.attribute(i).name());
      }
      
      m_graphTargetAtStepsCombo.setModel(new DefaultComboBoxModel(candidates));
      m_outputPredsCombo.setModel(new DefaultComboBoxModel(candidates));
    }
  }
  
  protected void updateDateDerivedPanel() {
    if (m_simpleConfig.m_timeStampCombo.getSelectedItem() == null) {
      return;
    }
    String selectedTimeStamp = m_simpleConfig.m_timeStampCombo.getSelectedItem().toString();
    if (selectedTimeStamp.equals("<None>") || 
        selectedTimeStamp.equals("<Use and artificial time stamp>")) {
      m_customizeDateDerivedPeriodics.setSelected(false);
      m_customizeDateDerivedPeriodics.setEnabled(false);
    } else if (m_instances != null) {
      Attribute timeStampAtt = m_instances.attribute(selectedTimeStamp);
      if (timeStampAtt != null && timeStampAtt.isDate()) {
        m_customizeDateDerivedPeriodics.setEnabled(true);
      } else {
        m_customizeDateDerivedPeriodics.setSelected(false);
        m_customizeDateDerivedPeriodics.setEnabled(false);
      }
    } else {
      m_customizeDateDerivedPeriodics.setSelected(false);
      m_customizeDateDerivedPeriodics.setEnabled(false);
    }
  }
  
  protected void updatePrimaryPeriodic() {
    Vector<String> entries = new Vector<String>();
    
    entries.add("<None>");
    if (m_instances != null) {
      for (int i = 0; i < m_instances.numAttributes(); i++) {
        if (m_instances.attribute(i).isNominal()) {
          entries.add(m_instances.attribute(i).name());
        }
      }
    }
    m_primaryPeriodicCombo.setModel(new DefaultComboBoxModel(entries));
  }
  
  protected void updateEvalAndOutputEnabledStatus() {
    boolean enable = (m_trainingCheckBox.isSelected() || 
        m_holdoutCheckBox.isSelected() ||
        m_separateTestSetCheckBox.isSelected()); 

    // enable/disable and deselect all prediction and graphing widgets
    m_outputPredsCheckBox.setEnabled(enable);
    
    //m_outputFutureCheckBox.setEnabled(enable);
    
    m_graphPredsAtStepCheckBox.setEnabled(enable);
    //m_outputStepSpinner.setEnabled(enable);
    //m_graphPredsAtStepSpinner.setEnabled(enable);
    m_graphTargetForStepsCheckBox.setEnabled(enable);
    
    //m_graphFutureCheckBox.setEnabled(enable);
    
    if (!enable) {
      m_outputPredsCheckBox.setSelected(false);
    //  m_outputFutureCheckBox.setSelected(false);
      m_graphPredsAtStepCheckBox.setSelected(false);
      m_graphTargetForStepsCheckBox.setSelected(false);
      
      
      //m_graphFutureCheckBox.setSelected(false);
    }
    
    boolean enabled = m_graphTargetForStepsCheckBox.isSelected();
    m_stepLab.setEnabled(enabled); m_stepRange.setEnabled(enabled);
    m_targetComboLabel.setEnabled(enabled); m_graphTargetAtStepsCombo.setEnabled(enabled);
    m_graphPredsAtStepSpinner.setEnabled(enabled);
    
    enabled = m_outputPredsCheckBox.isSelected();
    m_outputStepLabel.setEnabled(enabled); m_outputPredsCombo.setEnabled(enabled);
    m_outputPredsComboLabel.setEnabled(enable); m_outputStepSpinner.setEnabled(enabled);

        
    m_testSetBut.setEnabled(!m_holdoutCheckBox.isSelected());
    m_separateTestSetCheckBox.setEnabled(!m_holdoutCheckBox.isSelected());
    m_holdoutSize.setEnabled(m_holdoutCheckBox.isSelected());
    if (m_holdoutCheckBox.isSelected()) {
      m_separateTestSetCheckBox.setSelected(false);
    }
  }
  
  protected void layoutEvaluationPanel() {
    JPanel basePanel = new JPanel();
    basePanel.setLayout(new BorderLayout());
    /*basePanel.setBorder(BorderFactory.
        createTitledBorder("Evaluation")); */
    
    JPanel base1 = new JPanel();
    base1.setLayout(new BorderLayout());
    base1.add(m_evaluationMetrics, BorderLayout.CENTER);
    List<TSEvalModule> evalModulesL = TSEvalModule.getModuleList();
    ArrayList<Attribute> atts = new ArrayList<Attribute>();
    int numMods = 0;
    for (TSEvalModule s : evalModulesL) {
      if (!(s.getEvalName().equals("Error"))) {
        atts.add(new Attribute(s.toString()));
        numMods++;
      }
    }
    Instances modInsts = new Instances("Eval modules", atts, 1);
    m_evaluationModsHeader = modInsts;
    m_evaluationMetrics.setInstances(modInsts);
    boolean[] selected = new boolean[numMods];
    selected[0] = true; selected[2] = true;
    try {
      m_evaluationMetrics.setSelectedAttributes(selected);
    } catch (Exception ex) {}
    m_evaluationMetrics.setPreferredScrollableViewportSize(new Dimension(260,80));
    m_evaluationMetrics.setBorder(BorderFactory.createTitledBorder("Metrics"));
    
    JPanel temp1 = new JPanel();
    temp1.setLayout(new BorderLayout());
    temp1.setBorder(BorderFactory.createTitledBorder("Test options"));
    temp1.add(m_trainingCheckBox, BorderLayout.NORTH);
    //m_trainingCheckBox.setSelected(true);
    m_trainingCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        updateEvalAndOutputEnabledStatus();
        if (m_simpleConfig != null) {
          m_simpleConfig.m_performEvaluation.
            setSelected(m_trainingCheckBox.isSelected() ||
                m_holdoutCheckBox.isSelected());
        }
      }
    });
    
    JPanel temp2 = new JPanel();
    temp2.setLayout(new BorderLayout());
    temp2.add(m_holdoutCheckBox, BorderLayout.WEST);
    temp2.add(m_holdoutSize, BorderLayout.EAST);
    m_holdoutSize.setEnabled(false);
    
    int width = m_minLagSpinner.getPreferredSize().width;
    int height = m_minLagSpinner.getPreferredSize().height;
    m_holdoutSize.setPreferredSize(new Dimension(width * 1, height));
    m_holdoutSize.setMinimumSize(new Dimension(width * 1, height));
    m_holdoutSize.setToolTipText("Number of instances (value >=1) or percentage " +
    		"(value < 1) to hold out from the end of the data");
    m_holdoutSize.setText("0.3");
    m_holdoutCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        updateEvalAndOutputEnabledStatus();
        if (m_simpleConfig != null) {
          m_simpleConfig.m_performEvaluation.
            setSelected(m_trainingCheckBox.isSelected() ||
                m_holdoutCheckBox.isSelected());
        }
      }
    });
    temp1.add(temp2, BorderLayout.CENTER);
    
    if (m_allowSeparateTestSet) {
      JPanel checkAndButHolder = new JPanel();
      checkAndButHolder.setLayout(new BorderLayout());
      checkAndButHolder.add(m_separateTestSetCheckBox, BorderLayout.NORTH);
      checkAndButHolder.add(m_testSetBut, BorderLayout.SOUTH);
      temp1.add(checkAndButHolder, BorderLayout.SOUTH);
      m_separateTestSetCheckBox.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          updateEvalAndOutputEnabledStatus();
        }
      });
    }
                
    JPanel temp3 = new JPanel();
    temp3.setLayout(new BorderLayout());
    temp3.add(temp1, BorderLayout.NORTH);
    base1.add(temp3, BorderLayout.EAST);
    basePanel.add(base1, BorderLayout.NORTH);
    //basePanel.add(outputOptsHolder, BorderLayout.EAST);
    
    
    
    m_testSetBut.setToolTipText("Evaluate on a separate test set");
    m_testSetBut.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // TODO
      }
    });
    
    //holder.add(basePanel, BorderLayout.EAST);
    m_configHolder.addTab("Evaluation", null, basePanel, "Evaluation options");
  }
  
  protected void layoutOutputPanel() {
    JPanel outputOptsHolder = new JPanel();
    outputOptsHolder.setLayout(new GridLayout(1,2));    

    JPanel textOutHolder = new JPanel();
    textOutHolder.setLayout(new BorderLayout());
    textOutHolder.setBorder(BorderFactory.createTitledBorder("Output options"));
    
    SpinnerNumberModel snm = new SpinnerNumberModel();
    snm.setValue(1); snm.setMinimum(1);
    m_outputStepSpinner = new JSpinner(snm);
    Dimension spinD = m_outputStepSpinner.getPreferredSize();
    spinD = new Dimension((int)(spinD.getWidth() * 1.5), 
        (int)spinD.getHeight());
    m_outputStepSpinner.setPreferredSize(spinD);
    JPanel checkHolder = new JPanel();
    checkHolder.setLayout(new BorderLayout());
    checkHolder.add(m_outputPredsCheckBox, BorderLayout.WEST);
    //labAndSpinnerHolder.add(m_outputStepSpinner, BorderLayout.EAST);
    JPanel comboAndSpinnerHolder = new JPanel();
    comboAndSpinnerHolder.setLayout(new BorderLayout());
    
    JPanel combo1Holder = new JPanel(); combo1Holder.setLayout(new BorderLayout());
    combo1Holder.add(m_outputPredsCombo, BorderLayout.EAST);

    combo1Holder.add(m_outputPredsComboLabel, BorderLayout.CENTER);
    
    comboAndSpinnerHolder.add(combo1Holder, BorderLayout.NORTH);
    
    JPanel spinnerHolder1 = new JPanel(); 
    spinnerHolder1.setLayout(new BorderLayout());
    spinnerHolder1.add(m_outputStepSpinner, BorderLayout.EAST);
    spinnerHolder1.add(m_outputStepLabel, BorderLayout.CENTER);
    
    JPanel spinnerHolder2 = new JPanel();
    spinnerHolder2.setLayout(new BorderLayout());
    spinnerHolder2.add(spinnerHolder1, BorderLayout.NORTH);
    
    comboAndSpinnerHolder.add(spinnerHolder2, BorderLayout.CENTER);
    textOutHolder.add(comboAndSpinnerHolder, BorderLayout.CENTER);
    
    textOutHolder.add(checkHolder, BorderLayout.NORTH);
    m_outputStepSpinner.setEnabled(false);
    m_outputStepLabel.setEnabled(false);
    m_outputPredsCombo.setEnabled(false);
    m_outputPredsCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        m_outputStepSpinner.setEnabled(m_outputPredsCheckBox.isSelected());
        m_outputStepLabel.setEnabled(m_outputPredsCheckBox.isSelected());
        m_outputPredsComboLabel.setEnabled(m_outputPredsCheckBox.isSelected());
        m_outputPredsCombo.setEnabled(m_outputPredsCheckBox.isSelected());
      }
    });
    
    JPanel temp1 = new JPanel();
    temp1.setLayout(new BorderLayout());
    temp1.add(m_outputFutureCheckBox, BorderLayout.NORTH);
    textOutHolder.add(temp1, BorderLayout.SOUTH);
    outputOptsHolder.add(textOutHolder);
    m_outputFutureCheckBox.setSelected(true);
    
    JPanel graphOutputHolder = new JPanel();
    graphOutputHolder.setLayout(new BorderLayout());
    graphOutputHolder.setBorder(BorderFactory.
        createTitledBorder("Graphing options"));
    
    outputOptsHolder.add(graphOutputHolder);
    
    snm = new SpinnerNumberModel();
    snm.setValue(1); snm.setMinimum(1);
    m_graphPredsAtStepSpinner = new JSpinner(snm);
    spinD = m_graphPredsAtStepSpinner.getPreferredSize();
    spinD = new Dimension((int)(spinD.getWidth() * 1.5), 
        (int)spinD.getHeight());
    m_graphPredsAtStepSpinner.setPreferredSize(spinD);
    JPanel labAndSpinnerHolder = new JPanel();
    labAndSpinnerHolder.setLayout(new BorderLayout());
    labAndSpinnerHolder.add(m_graphPredsAtStepCheckBox, BorderLayout.WEST);
    labAndSpinnerHolder.add(m_graphPredsAtStepSpinner, BorderLayout.EAST);
    graphOutputHolder.add(labAndSpinnerHolder, BorderLayout.NORTH);
    m_graphPredsAtStepCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        m_graphPredsAtStepSpinner.
          setEnabled(m_graphPredsAtStepCheckBox.isSelected());
      }
    });
    m_graphPredsAtStepSpinner.setEnabled(false);
    
    JPanel comboAndRangeHolder = new JPanel();
    comboAndRangeHolder.setLayout(new BorderLayout());
    comboAndRangeHolder.add(m_graphTargetForStepsCheckBox, BorderLayout.NORTH);
    JPanel temp2 = new JPanel();
    temp2.setLayout(new BorderLayout());
    temp2.add(m_graphTargetAtStepsCombo, BorderLayout.EAST);

    temp2.add(m_targetComboLabel, BorderLayout.CENTER);
    JPanel temp3 = new JPanel();
    temp3.setLayout(new BorderLayout());
    temp3.add(temp2, BorderLayout.NORTH);
    
    JPanel textHolder = new JPanel();
    textHolder.setLayout(new BorderLayout());
    textHolder.add(m_stepRange, BorderLayout.EAST);
    m_stepRange.setPreferredSize(new Dimension((int)spinD.getWidth() * 2,
        (int)spinD.getHeight()));
    
    m_stepLab.setToolTipText("Comma separated list of step numbers to graph");
    m_stepRange.setToolTipText("Comma separated list of step numbers to graph");
    JPanel ll = new JPanel(); ll.setLayout(new BorderLayout());
    ll.add(m_stepLab, BorderLayout.EAST);    

    textHolder.add(ll, BorderLayout.CENTER);
    temp3.add(textHolder, BorderLayout.CENTER);

    JPanel ll2 = new JPanel(); ll2.setLayout(new BorderLayout());
    ll2.add(temp3, BorderLayout.NORTH);
    comboAndRangeHolder.add(ll2, BorderLayout.CENTER);        
    comboAndRangeHolder.add(m_graphFutureCheckBox, BorderLayout.SOUTH);
    m_graphFutureCheckBox.setSelected(true);
    
    m_stepLab.setEnabled(false); m_stepRange.setEnabled(false);
    m_targetComboLabel.setEnabled(false); m_graphTargetAtStepsCombo.setEnabled(false);
    m_graphTargetForStepsCheckBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        boolean enabled = m_graphTargetForStepsCheckBox.isSelected();
        m_stepLab.setEnabled(enabled); m_stepRange.setEnabled(enabled);
        m_targetComboLabel.setEnabled(enabled); m_graphTargetAtStepsCombo.setEnabled(enabled);
      }
    });
    
    graphOutputHolder.add(comboAndRangeHolder, BorderLayout.CENTER);
    
    m_configHolder.addTab("Output", null, outputOptsHolder, "Configure output");
  }
  
  protected void layoutDateDerivedPeriodicPanel() {
    JPanel basePanel = new JPanel();
    basePanel.setLayout(new BorderLayout());        
    
    JPanel temp = new JPanel();
    temp.setLayout(new BorderLayout());
    //temp
    JPanel checkHolder = new JPanel();
    checkHolder.setLayout(new BorderLayout());
    checkHolder.add(m_customizeDateDerivedPeriodics, BorderLayout.EAST);
    temp.add(checkHolder, BorderLayout.NORTH);
    temp.add(m_dateDerivedPeriodicSelector, BorderLayout.CENTER);
    m_dateDerivedPeriodicSelector.setPreferredScrollableViewportSize(new Dimension(250,80));
    JPanel botP = new JPanel();
    botP.setLayout(new BorderLayout());
    botP.setBorder(BorderFactory.
        createTitledBorder("Date-derived periodic creation"));
    botP.add(temp, BorderLayout.CENTER);
    
    JPanel primaryPeriodicP = new JPanel();
    primaryPeriodicP.setLayout(new BorderLayout());
    //primaryPeriodicP
      
    primaryPeriodicP.add(m_primaryPeriodicCombo, BorderLayout.NORTH);
    JPanel topP = new JPanel();
    topP.setLayout(new BorderLayout());
    topP.setBorder(BorderFactory.createTitledBorder("Periodic attribute"));
    topP.add(primaryPeriodicP, BorderLayout.EAST);
    
    JPanel temp2 = new JPanel();
    temp2.setLayout(new BorderLayout());
    temp2.add(topP, BorderLayout.EAST);
    temp2.add(botP, BorderLayout.CENTER);
//    basePanel.add(primaryPeriodicP, BorderLayout.EAST);
    basePanel.add(temp2, BorderLayout.NORTH);
    
    m_customizeDateDerivedPeriodics.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (m_instances != null && m_customizeDateDerivedPeriodics.isSelected()) {
          String selectedTimeStamp = 
            m_simpleConfig.m_timeStampCombo.getSelectedItem().toString();
          Attribute timeStampAtt = m_instances.attribute(selectedTimeStamp);
          
          if (timeStampAtt != null && timeStampAtt.isDate()) {
            if (m_dateDerivedPeriodicSelector.getSelectionModel() != null) {
              ArrayList<Attribute> atts = new ArrayList<Attribute>();
              atts.add(new Attribute("AM")); atts.add(new Attribute("DayOfWeek"));
              atts.add(new Attribute("Weekend")); atts.add(new Attribute("Month"));
              atts.add(new Attribute("Quarter"));
              Instances insts = new Instances("Periodics", atts, 1);
              m_dateDerivedPeriodicSelector.setInstances(insts);
              m_dateDerivedPeriodicsHeader = insts;
            }
          }
        }
      }
    });
    
    updatePrimaryPeriodic();
        
    //holder.add(basePanel, BorderLayout.WEST);
    m_configHolder.addTab("Periodic attributes", null, basePanel,
        "Specify/clustomize periodic attributes");
    updateDateDerivedPanel();
  }
  
  protected void layoutLearnerPanel() {
    JPanel basePanel = new JPanel();
    basePanel.setLayout(new BorderLayout());
    basePanel.setBorder(BorderFactory.
        createTitledBorder("Base learner configuration"));
    
    m_baseLearnerEditor.setClassType(Classifier.class);
    m_baseLearnerEditor.setValue(new weka.classifiers.functions.SMOreg());
    
    Capabilities capabilities = new Capabilities(null);
    capabilities.disableAll();
    capabilities.enable(Capability.NOMINAL_ATTRIBUTES);
    capabilities.enable(Capability.NUMERIC_ATTRIBUTES);
    capabilities.enable(Capability.NUMERIC_CLASS);
    capabilities.enableAllAttributeDependencies();
    capabilities.enableAllClassDependencies();
    m_baseLearnerEditor.setCapabilitiesFilter(capabilities);
    
    // TODO add capabilities filtering here
    basePanel.add(m_baseLearnerPanel, BorderLayout.NORTH);
    //add(basePanel, BorderLayout.NORTH);
    m_configHolder.addTab("Base learner", null, basePanel, 
        "Base learner configuration");
  }
  
  protected void layoutLagPanel() {
    // lag stuff
    JPanel lagPanel = new JPanel();
    lagPanel.setLayout(new GridLayout(1,2));
    //lagPanel.setBorder(BorderFactory.createTitledBorder("Lag creation"));    
    
    SpinnerNumberModel snm = new SpinnerNumberModel();
    snm.setValue(1); snm.setMinimum(1);
    m_minLagSpinner = new JSpinner(snm);
    Dimension spinD = m_minLagSpinner.getPreferredSize();
    spinD = new Dimension((int)(spinD.getWidth() * 1.5), 
        (int)spinD.getHeight());
    m_minLagSpinner.setPreferredSize(spinD);
    
    JPanel temp1 = new JPanel();
    temp1.setLayout(new BorderLayout());
    final JLabel minLagLab = new JLabel("Minimum lag", JLabel.RIGHT);
    temp1.add(minLagLab, BorderLayout.CENTER);
    temp1.add(m_minLagSpinner, BorderLayout.EAST);
    JPanel spinnerHolder = new JPanel();
    spinnerHolder.setLayout(new BorderLayout());
    spinnerHolder.setBorder(BorderFactory.createTitledBorder("Lag length"));
    
    JPanel vH = new JPanel();
    vH.setLayout(new BorderLayout());

    JPanel varianceHolder = new JPanel();
    varianceHolder.setLayout(new BorderLayout());
    varianceHolder.add(m_adjustForVarianceCheckBox, BorderLayout.EAST);
    m_adjustForVarianceCheckBox.setSelected(false);
    vH.add(varianceHolder, BorderLayout.NORTH);
    
    JPanel checkHolder = new JPanel();
    checkHolder.setLayout(new BorderLayout());
    checkHolder.add(m_useCustomLags, BorderLayout.EAST);
    vH.add(checkHolder, BorderLayout.SOUTH);
    
    spinnerHolder.add(vH, BorderLayout.NORTH);
    spinnerHolder.add(temp1, BorderLayout.CENTER);
    
    snm = new SpinnerNumberModel();
    snm.setValue(12); snm.setMinimum(1);
    m_maxLagSpinner = new JSpinner(snm);
    m_maxLagSpinner.setPreferredSize(spinD);
    JPanel temp2 = new JPanel();
    temp2.setLayout(new BorderLayout());
    final JLabel maxLagLab = new JLabel("Maximum lag", JLabel.RIGHT);
    temp2.add(maxLagLab, BorderLayout.CENTER);
    temp2.add(m_maxLagSpinner, BorderLayout.EAST);
    
    final JLabel fineTuneLab = new JLabel("Fine tune lag selection", JLabel.RIGHT);
    JPanel fPanel = new JPanel(); fPanel.setLayout(new BorderLayout());
    fPanel.add(temp2, BorderLayout.NORTH);
    
    JPanel fineHolder = new JPanel(); fineHolder.setLayout(new BorderLayout());
    fineHolder.add(fineTuneLab, BorderLayout.CENTER);
    fineHolder.add(m_fineTuneLagsField, BorderLayout.EAST);
    m_fineTuneLagsField.setPreferredSize(spinD);
    fineTuneLab.setEnabled(false);
    fineTuneLab.setToolTipText("Specify ranges to fine tune " +
    		"lags within minimum and maximum (e.g. 2,3,6-8)");
    m_fineTuneLagsField.setEnabled(false);
    m_fineTuneLagsField.setToolTipText("Specify ranges to fine tune " +
        "lags within minimum and maximum (e.g. 2,3,6-8)");
    
    fPanel.add(fineHolder,BorderLayout.SOUTH);
            
    spinnerHolder.add(fPanel, BorderLayout.SOUTH);
    
    m_useCustomLags.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        boolean enabled = m_useCustomLags.isSelected();
        m_minLagSpinner.setEnabled(enabled);
        m_maxLagSpinner.setEnabled(enabled);
        minLagLab.setEnabled(enabled);
        maxLagLab.setEnabled(enabled);
        fineTuneLab.setEnabled(enabled);
        m_fineTuneLagsField.setEnabled(enabled);
        
        m_averageLongLags.setEnabled(enabled);
        if (!enabled) {
          m_averageLongLags.setSelected(false);
        }
        /*m_averageLagsAfter.setEnabled(enabled);
        m_numConsecutiveToAverage.setEnabled(enabled); */
      }
    });    
    
    snm = new SpinnerNumberModel();
    snm.setValue(2); snm.setMinimum(1);
    m_averageLagsAfter = new JSpinner(snm);
    m_averageLagsAfter.setPreferredSize(spinD);
    
    snm = new SpinnerNumberModel();
    snm.setValue(2); snm.setMinimum(2);
    m_numConsecutiveToAverage = new JSpinner(snm);
    m_numConsecutiveToAverage.setPreferredSize(spinD);
    
    JPanel averageLagHolder = new JPanel();
    averageLagHolder.setBorder(BorderFactory.createTitledBorder("Averaging"));
    averageLagHolder.setLayout(new BorderLayout());
    JPanel avCheckHolder = new JPanel();
    avCheckHolder.setLayout(new BorderLayout());
    avCheckHolder.add(m_averageLongLags, BorderLayout.EAST);
    averageLagHolder.add(avCheckHolder, BorderLayout.NORTH);
    JPanel temp3 = new JPanel();
    temp3.setLayout(new BorderLayout());
    final JLabel avLab =new JLabel("Average lags longer than", JLabel.RIGHT); 
    temp3.add(avLab, BorderLayout.CENTER);
    temp3.add(m_averageLagsAfter, BorderLayout.EAST);
    
    JPanel aH = new JPanel();
    aH.setLayout(new BorderLayout());
    aH.add(temp3, BorderLayout.NORTH);
    
    //averageLagHolder.add(temp3, BorderLayout.CENTER);
    JPanel temp4 = new JPanel();
    temp4.setLayout(new BorderLayout());
    final JLabel numConsLab = new JLabel("# consecutive lags to average", JLabel.RIGHT); 
    temp4.add(numConsLab, BorderLayout.CENTER);
    temp4.add(m_numConsecutiveToAverage, BorderLayout.EAST);
    aH.add(temp4, BorderLayout.SOUTH);
    averageLagHolder.add(aH, BorderLayout.SOUTH);
    
    m_useCustomLags.setSelected(false);
    m_minLagSpinner.setEnabled(false);
    m_maxLagSpinner.setEnabled(false);
    minLagLab.setEnabled(false);
    maxLagLab.setEnabled(false);
    m_averageLongLags.setEnabled(false);
    avLab.setEnabled(false);
    m_averageLagsAfter.setEnabled(false);
    m_numConsecutiveToAverage.setEnabled(false);
    numConsLab.setEnabled(false);
    
    m_averageLongLags.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (m_useCustomLags.isSelected()) {
          boolean enabled = m_averageLongLags.isSelected();
          m_averageLagsAfter.setEnabled(enabled);
          m_numConsecutiveToAverage.setEnabled(enabled);
          avLab.setEnabled(enabled);
          numConsLab.setEnabled(enabled);
        }
      }
    });
        
    lagPanel.add(spinnerHolder);//, BorderLayout.CENTER);
    lagPanel.add(averageLagHolder);//, BorderLayout.EAST);
    
    JPanel tempP = new JPanel();
    tempP.setLayout(new BorderLayout());
    tempP.add(lagPanel, BorderLayout.NORTH);
    
    
            
    //add(tempP, BorderLayout.CENTER);
    m_configHolder.addTab("Lag creation", null, tempP, 
        "Customize lagged variables");
  }
  
  public void applyToForecaster(WekaForecaster forecaster) throws Exception {
    if (forecaster != null) {
      TSLagMaker lagMaker = forecaster.getTSLagMaker();
      
      // Set variance adjustment
      lagMaker.setAdjustForVariance(m_adjustForVarianceCheckBox.isSelected());
      
      int minLag = ((SpinnerNumberModel)m_minLagSpinner.getModel()).
        getNumber().intValue();
      int maxLag = ((SpinnerNumberModel)m_maxLagSpinner.getModel()).
        getNumber().intValue();
      if (m_useCustomLags.isSelected()) {
        
        // set lag lengths from spinners                
        if (maxLag < minLag) {
          throw new Exception("Maximum lag value must be greater than or equal to" +
          		" the minimum lag value");
        }
        
        lagMaker.setMinLag(minLag); lagMaker.setMaxLag(maxLag);
        
        if (m_fineTuneLagsField.getText() != null && 
            m_fineTuneLagsField.getText().length() > 0) {
          lagMaker.setLagRange(m_fineTuneLagsField.getText());
        }
        
        // set up averaging from spinners
        if (m_averageLongLags.isSelected()) {
          lagMaker.setAverageConsecutiveLongLags(true);
          
          int avLagsAfter = ((SpinnerNumberModel)m_averageLagsAfter.getModel()).
            getNumber().intValue();
          int numToAv = ((SpinnerNumberModel)m_numConsecutiveToAverage.getModel()).
            getNumber().intValue();
          
          if (avLagsAfter < minLag || avLagsAfter > maxLag) {
            throw new Exception("Point at which to start lag averaging must " +
            		"lie between the minimum and maximum lag value.");
          }
          
          lagMaker.setAverageLagsAfter(avLagsAfter);
          lagMaker.setNumConsecutiveLongLagsToAverage(numToAv);
        } else {
          lagMaker.setAverageConsecutiveLongLags(false);
        }
      } else {
        lagMaker.setAverageConsecutiveLongLags(false);
        lagMaker.setLagRange("");
      }
      
      // date-derived periodic customization
      if (getCustomizeDateDerivedPeriodics()) {
        // reset all to false
        lagMaker.setAddAMIndicator(false); lagMaker.setAddDayOfWeek(false);
        lagMaker.setAddMonthOfYear(false); lagMaker.setAddQuarterOfYear(false);
        lagMaker.setAddWeekendIndicator(false);
        
        int[] selected = m_dateDerivedPeriodicSelector.getSelectedAttributes();
        if (m_dateDerivedPeriodicsHeader != null) {
          for (int s : selected) {
            String name = m_dateDerivedPeriodicsHeader.attribute(s).name();
            if (name.equals("AM")) {
              lagMaker.setAddAMIndicator(true);              
            }
            if (name.equals("DayOfWeek")) {
              lagMaker.setAddDayOfWeek(true);
            }
            if (name.equals("Weekend")) {
              lagMaker.setAddWeekendIndicator(true);
            }
            if (name.equals("Month")) {
              lagMaker.setAddMonthOfYear(true);
            }
            if (name.equals("Quarter")) {
              lagMaker.setAddQuarterOfYear(true);
            }
          }
        }
      }
      
      // non-date primary periodic attribute
      String ppfn = m_primaryPeriodicCombo.getSelectedItem().toString();
      if (!ppfn.equals("<None>")) {
        lagMaker.setPrimaryPeriodicFieldName(ppfn);
      } else {
        lagMaker.setPrimaryPeriodicFieldName("");
      }
    }
  }
  
  public void applyToEvaluation(TSEvaluation eval, WekaForecaster forecaster) 
    throws Exception {
    
    // eval on training
    eval.setEvaluateOnTrainingData(m_trainingCheckBox.isSelected());
    
    if (!eval.getEvaluateOnTrainingData() && !eval.getEvaluateOnTestData()) {
      // throw new Exception("Must evaluate")
    }
    
    // evaluation modules
    int[] selected = m_evaluationMetrics.getSelectedAttributes();
    
    if (selected.length == 0) {
      throw new Exception("Must select at least one evaluation metric.");
    }
    
    List<TSEvalModule> modsList = TSEvalModule.getModuleList();
    String modListS = "";
    for (int s : selected) {
      String name = m_evaluationModsHeader.attribute(s).name();
      for (TSEvalModule mod : modsList) {
        if (name.equals(mod.toString())) {
          modListS += mod.getEvalName() + ",";
        }
      }
    }
    
    modListS = modListS.substring(0, modListS.lastIndexOf(','));
    eval.setEvaluationModules(modListS);
    
    // TODO separate test set
  }
  
  /**
   * Tests the Weka advanced config panel from the command line.
   *
   * @param args must contain the name of an arff file to load.
   */
  public static void main(String[] args) {

    try {
      if (args.length == 0) {
        throw new Exception("supply the name of an arff file");
      }
      Instances i = new Instances(new java.io.BufferedReader(
                                  new java.io.FileReader(args[0])));
      SimpleConfigPanel scp = new SimpleConfigPanel(null);
      scp.setInstances(i);
      AdvancedConfigPanel acp = new AdvancedConfigPanel(scp);
      //acp.setInstances(i);
      
      final javax.swing.JFrame jf =
        new javax.swing.JFrame("Weka Forecasting");
      jf.getContentPane().setLayout(new BorderLayout());
      jf.getContentPane().add(acp, BorderLayout.CENTER);
      jf.addWindowListener(new java.awt.event.WindowAdapter() {
        public void windowClosing(java.awt.event.WindowEvent e) {
          jf.dispose();
          System.exit(0);
        }
      });
      jf.pack();
      jf.setVisible(true);
    } catch (Exception ex) {
      ex.printStackTrace();
      System.err.println(ex.getMessage());
    }
  }
}
