package weka.classifiers.timeseries.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;

import weka.classifiers.timeseries.TSLagMaker;
import weka.classifiers.timeseries.WekaForecaster;
import weka.classifiers.timeseries.eval.TSEvaluation;
import weka.core.Attribute;
import weka.core.Instances;
import weka.filters.Filter;
import weka.filters.unsupervised.attribute.Remove;
import weka.gui.AttributeSelectionPanel;

public class SimpleConfigPanel extends JPanel {
  
  /** The training instances to operate on */
  protected Instances m_instances;
  
  /** The forecaster to configure */
  protected WekaForecaster m_forecaster;
  
  /** 
   * Holds the header of the training instances after all but
   * numeric attributes are removed (for target selection 
   * purposes
   */
  protected Instances m_targetHeader;
  
  protected JComboBox m_timeStampCombo = new JComboBox();
  protected JComboBox m_periodicityCombo = new JComboBox();
  protected AttributeSelectionPanel m_targetPanel = new AttributeSelectionPanel();
  protected JCheckBox m_computeConfidence = new JCheckBox();
  protected JSpinner m_horizonSpinner;
  
  protected JSpinner m_confidenceLevelSpinner;
  
  protected JCheckBox m_performEvaluation = new JCheckBox();
  
  protected AdvancedConfigPanel m_advancedConfig;
  protected ForecastingPanel m_parentPanel;
  
  public SimpleConfigPanel(ForecastingPanel parent) {
    
    m_parentPanel = parent;
    setLayout(new BorderLayout());
    
    JPanel colSelect = new JPanel();
    colSelect.setLayout(new BorderLayout());
    colSelect.setBorder(BorderFactory.createTitledBorder("Target Selection " +
    		"and Parameters"));
    JPanel tempHolder1 = new JPanel();
    tempHolder1.setLayout(new BorderLayout());
    tempHolder1.add(m_targetPanel, BorderLayout.NORTH);
    colSelect.add(tempHolder1, BorderLayout.NORTH);
    
    m_targetPanel.setPreferredScrollableViewportSize(new Dimension(250,80));
    SpinnerNumberModel snm = new SpinnerNumberModel();
    snm.setValue(1);
    snm.setMinimum(1);
    m_horizonSpinner = new JSpinner(snm);
    Dimension spinD = m_horizonSpinner.getPreferredSize();
    spinD = new Dimension((int)(spinD.getWidth() * 1.5), 
        (int)spinD.getHeight());
    m_horizonSpinner.setPreferredSize(spinD);
    JPanel spinnerHolder = new JPanel();
    spinnerHolder.setLayout(new BorderLayout());
    spinnerHolder.add(m_horizonSpinner, BorderLayout.EAST);
    spinnerHolder.add(new JLabel("Number of time units to forecast ", JLabel.RIGHT), 
        BorderLayout.CENTER);
    
    tempHolder1.add(spinnerHolder, BorderLayout.CENTER);
    JPanel spacer = new JPanel();
    spacer.setMinimumSize(spinD);
    tempHolder1.add(spacer, BorderLayout.SOUTH);
    
    add(colSelect, BorderLayout.CENTER);
    Box comboHolder = new Box(BoxLayout.PAGE_AXIS);
    JPanel timeHolder = new JPanel();
    timeHolder.setLayout(new BorderLayout());
    //timeHolder.setBorder(BorderFactory.createTitledBorder("Time stamp"));
    timeHolder.add(new JLabel("Time stamp ", JLabel.RIGHT), BorderLayout.WEST);
    timeHolder.add(m_timeStampCombo, BorderLayout.EAST);
    comboHolder.add(timeHolder);
    JPanel periodicityHolder = new JPanel();
    periodicityHolder.setLayout(new BorderLayout());
//    periodicityHolder.setBorder(BorderFactory.createTitledBorder("Periodicity"));
    periodicityHolder.add(new JLabel("Periodicity", JLabel.RIGHT), BorderLayout.WEST);    
    periodicityHolder.add(m_periodicityCombo, BorderLayout.EAST);
    comboHolder.add(periodicityHolder);
    m_periodicityCombo.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        checkPeriodicity(null);
      }
    });
    
    m_computeConfidence.setHorizontalTextPosition(SwingConstants.LEFT);
    
    JPanel confHolder = new JPanel();
    confHolder.setLayout(new BorderLayout());
    confHolder.add(new JLabel("Confidence intervals", JLabel.RIGHT), BorderLayout.WEST);
    confHolder.add(m_computeConfidence, BorderLayout.EAST);
    comboHolder.add(confHolder);
    
    JPanel confLevHolder = new JPanel();
    confLevHolder.setLayout(new BorderLayout());
    final JLabel levelLab = new JLabel("Level (%) ", JLabel.RIGHT);
    levelLab.setEnabled(false);
    snm = new SpinnerNumberModel();
    snm.setValue(95);
    snm.setMinimum(1);
    snm.setMaximum(99);
    m_confidenceLevelSpinner = new JSpinner(snm);
    m_confidenceLevelSpinner.setEnabled(false);
    confLevHolder.add(m_confidenceLevelSpinner, BorderLayout.EAST);
    confLevHolder.add(levelLab, BorderLayout.CENTER);
        
    comboHolder.add(confLevHolder);
    
    m_computeConfidence.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        boolean enable = m_computeConfidence.isSelected();
        levelLab.setEnabled(enable);
        m_confidenceLevelSpinner.setEnabled(enable);
      }
    });
    
    JPanel evalHolder = new JPanel();
    evalHolder.setLayout(new BorderLayout());
    evalHolder.add(new JLabel("Perform evaluation", JLabel.RIGHT), BorderLayout.WEST);
    m_performEvaluation.setHorizontalAlignment(SwingConstants.LEFT);
    evalHolder.add(m_performEvaluation, BorderLayout.EAST);
    comboHolder.add(evalHolder);
    m_performEvaluation.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        m_advancedConfig.m_trainingCheckBox.
          setSelected(m_performEvaluation.isSelected());
        // don't enable test set evaluation, only disable
        if (!m_performEvaluation.isSelected()) {
          m_advancedConfig.m_holdoutCheckBox.setSelected(false);
        }
      }
    });
    
    //comboHolder.add(m_horizon);
  
    JPanel temp = new JPanel();
    temp.setLayout(new BorderLayout());
    temp.setBorder(BorderFactory.createTitledBorder("Options"));
    
    temp.add(comboHolder, BorderLayout.NORTH);
    add(temp, BorderLayout.EAST);
    
    m_timeStampCombo.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (m_advancedConfig != null) {
          m_advancedConfig.updateDateDerivedPanel();
        }
      }
    });
    
    /*m_periodicityCombo.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (m_forecaster != null) {
          
        }
      }
    }); */
    
    //m_targetPanel
    /*Dimension d = tempHolder1.getPreferredSize();
    Dimension d2 = spinnerHolder.getPreferredSize();
    tempHolder1.setMinimumSize(new Dimension(tempHolder1.getPreferredSize().width, d.height + d2.height));
    tempHolder1.setPreferredSize(new Dimension(tempHolder1.getPreferredSize().width, d.height + d2.height));*/
  }
  
  public String getTabTitle() {
    return "Basic configuration";
  }
  
  public String getTabTitleToolTip() {
    return "Basic configuration";
  }
  
  public int getHorizonValue() {
    SpinnerNumberModel snm = (SpinnerNumberModel)m_horizonSpinner.getModel();
    
    return snm.getNumber().intValue();
  }
  
  public void setAdvancedConfig(AdvancedConfigPanel adv) {
    m_advancedConfig = adv;
  }
  
  public void setForecaster(WekaForecaster forecaster) {
    m_forecaster = forecaster;
    if (m_instances != null) {
      if (!m_forecaster.getTSLagMaker().getAdjustForTrends()) {
        m_timeStampCombo.setSelectedItem("<None>");
        m_periodicityCombo.setSelectedIndex(0);
      } else {        
        if (m_forecaster.getTSLagMaker().isUsingAnArtificialTimeIndex()) {
          m_timeStampCombo.setSelectedItem("<Use an artificial time index>");
        } else if (m_forecaster.getTSLagMaker().getTimeStampField() != null &&
            m_forecaster.getTSLagMaker().getTimeStampField().length() > 0) {
          m_timeStampCombo.
            setSelectedItem(m_forecaster.getTSLagMaker().getTimeStampField());
        }
      }      

      int maxLag = m_forecaster.getTSLagMaker().getMaxLag();
      if (maxLag == 24) {
        m_periodicityCombo.setSelectedItem("Hourly");
      }
      if (maxLag == 7) {
        m_periodicityCombo.setSelectedItem("Daily");
      }
      if (maxLag == 5) {
        m_periodicityCombo.setSelectedItem("Weekly");
      }
      if (maxLag == 12) {
        m_periodicityCombo.setSelectedItem("Monthly");
      }
      if (maxLag == 4) {
        m_periodicityCombo.setSelectedItem("Quarterly");
      }
      if (maxLag == 10) {
        m_periodicityCombo.setSelectedItem("Yearly");
      } else {
        m_periodicityCombo.setSelectedItem("<Unknown>");
      }
      
      // TODO read selected targets from forecaster and
      // set in list
    }    
  }
  
  public boolean isUsingANativeTimeStamp() {
    
    String selected = m_timeStampCombo.getSelectedItem().toString();
    
    if (!selected.equals("<None>") && 
        !selected.equals("<Use an artificial time index>")) {
      return true;
    }

    return false;
  }
  
  public String getSelectedTimeStampField() {
    return m_timeStampCombo.getSelectedItem().toString();
  }
  
  public WekaForecaster getForecaster() {
    return m_forecaster;
  }
  
  public void setHorizon(int horizon) {
    m_horizonSpinner.setValue(horizon);
  }
  
  public int getHorizon() {
    SpinnerNumberModel model = (SpinnerNumberModel)m_horizonSpinner.getModel();
    return model.getNumber().intValue();
  }
  
  public void setInstances(Instances insts) throws Exception {
    m_instances = insts;
    
    String removeList = "";
    for (int i = 0; i < m_instances.numAttributes(); i++){
      if (m_instances.attribute(i).isDate() || 
          m_instances.attribute(i).isNominal() ||
          m_instances.attribute(i).isString() ||
          m_instances.attribute(i).isRelationValued()) {
        removeList += (i + 1) + ",";
      }
    }
    m_targetHeader = new Instances(m_instances, 0);
    if (removeList.length() > 0) {
      removeList = removeList.substring(0, removeList.lastIndexOf(","));
      
      Remove r = new Remove();
      r.setAttributeIndices(removeList);
      r.setInputFormat(m_instances);
      m_targetHeader = Filter.useFilter(m_instances, r);
      m_targetHeader = new Instances(m_targetHeader, 0);
    }
    m_targetPanel.setInstances(m_targetHeader);
    if (m_targetHeader.numAttributes() == 1) {
      // auto select the only target
      m_targetPanel.setSelectedAttributes(new boolean[] {true});
      if (m_parentPanel != null) {
        m_parentPanel.enableStartButton(true);
      }
    }
    setupTimeCombo();
    setupPeriodicityCombo();    
  }
  
  protected void setupTimeCombo() {
    Vector<String> candidateNames = new Vector<String>();
    
    candidateNames.add("<Use an artificial time index>");
    candidateNames.add("<None>");
    String firstDateName = null;
    for (int i = 0; i < m_instances.numAttributes(); i++) {
      if (m_instances.attribute(i).isNumeric()) {
        candidateNames.add(m_instances.attribute(i).name());
      }
      if (m_instances.attribute(i).isDate() && firstDateName == null) {
        firstDateName = m_instances.attribute(i).name();        
      }
    }
    
    m_timeStampCombo.setModel(new DefaultComboBoxModel(candidateNames));
    if (firstDateName != null) {
      m_timeStampCombo.setSelectedItem(firstDateName);
    }
  }
  
  protected void setupPeriodicityCombo() {
    Vector<String> candidateNames = new Vector<String>();

    candidateNames.add("<Unknown>");
    candidateNames.add("<Detect automatically>");
    candidateNames.add("Hourly");
    candidateNames.add("Daily");
    candidateNames.add("Weekly");
    candidateNames.add("Monthly");
    candidateNames.add("Quarterly");
    candidateNames.add("Yearly");
    
    m_periodicityCombo.setModel(new DefaultComboBoxModel(candidateNames));
    
    // check to see if a date is set in the time combo and, if so, select
    // <detect automatically> by default
    String timeSelected = (String)m_timeStampCombo.getSelectedItem();
    System.err.println("Selected " + timeSelected);
    if (timeSelected != null && timeSelected.length() > 0 && 
        !timeSelected.equals("<None>") && 
        !timeSelected.equals("<Use an artificial time index>") &&
        m_instances.attribute(timeSelected).isDate()) {
      m_periodicityCombo.setSelectedItem("<Detect automatically>");
    }
  }
  
  protected void checkPeriodicity(WekaForecaster forecaster) {
    
    if (m_advancedConfig != null) {
      String timeStampSelected = m_timeStampCombo.getSelectedItem().toString();
      String selectedP = m_periodicityCombo.getSelectedItem().toString();
      if (selectedP.equals("<Detect automatically>") &&
          !timeStampSelected.equals("<Use an artificial time index>") &&
          !timeStampSelected.equals("<None>") &&
          forecaster != null) {
        TSLagMaker.Periodicity detected = 
          TSLagMaker.determinePeriodicity(m_instances, forecaster.
              getTSLagMaker().getTimeStampField());
        switch (detected) {
        case HOURLY:
          selectedP = "Hourly";
          break;
        case DAILY:
          selectedP = "Daily";
          break;
        case WEEKLY:
          selectedP = "Weekly";
          break;
        case MONTHLY:
          selectedP = "Monthly";
          break;
        case QUARTERLY:
          selectedP = "Quarterly";
          break;
        case YEARLY:
          selectedP = "Yearly";
          break;                
        }
      }

      if (selectedP.equals("Hourly")) {
        if (forecaster != null) {
          forecaster.getTSLagMaker().setMaxLag(24);
        }
        m_advancedConfig.m_maxLagSpinner.setValue(24);
      } else if (selectedP.equals("Daily")) {
        if (forecaster != null) {
          forecaster.getTSLagMaker().setMaxLag(7);
        }        
        m_advancedConfig.m_maxLagSpinner.setValue(7);
      } else if (selectedP.equals("Weekly")) {
        if (forecaster != null) {
          forecaster.getTSLagMaker().setMaxLag(52);
        }
        m_advancedConfig.m_maxLagSpinner.setValue(52);
      } else if (selectedP.equals("Monthly")) {
        if (forecaster != null) {
          forecaster.getTSLagMaker().setMaxLag(12);
        }
        m_advancedConfig.m_maxLagSpinner.setValue(12);
      } else if (selectedP.equals("Quarterly")) {
        if (forecaster != null) {
          forecaster.getTSLagMaker().setMaxLag(4);
        }
        m_advancedConfig.m_maxLagSpinner.setValue(4);
      } else if (selectedP.equals("Yearly")) {
        if (forecaster != null) {
          forecaster.getTSLagMaker().setMaxLag(10);
        }
        m_advancedConfig.m_maxLagSpinner.setValue(10);
      } else {
        // default
        if (forecaster != null) {
          forecaster.getTSLagMaker().setMaxLag(12);
        }
        m_advancedConfig.m_maxLagSpinner.setValue(12);
      }

      if (!m_advancedConfig.getCustomizeDateDerivedPeriodics() && forecaster != null) {
        // configure defaults based on the above periodicity
        if (selectedP.equals("Hourly")) {
          forecaster.getTSLagMaker().setAddAMIndicator(true);
        } else if (selectedP.equals("Daily")) {
          forecaster.getTSLagMaker().setAddDayOfWeek(true);
          forecaster.getTSLagMaker().setAddWeekendIndicator(true);
        } else if (selectedP.equals("Weekly")) {
          forecaster.getTSLagMaker().setAddMonthOfYear(true);
          forecaster.getTSLagMaker().setAddQuarterOfYear(true);
        } else if (selectedP.equals("Monthly")) {
          forecaster.getTSLagMaker().setAddMonthOfYear(true);
          forecaster.getTSLagMaker().setAddQuarterOfYear(true);
        }
      }
    }        
  }
  
  public void applyToForecaster(WekaForecaster forecaster) throws Exception {
    if (forecaster != null) {
      TSLagMaker lagMaker = forecaster.getTSLagMaker();
      
      String selected = 
        m_timeStampCombo.getSelectedItem().toString();
      if (!selected.equals("<Use an artificial time index>") && 
          !selected.equals("<None>")) {
        lagMaker.setTimeStampField(selected);
      } else {
        lagMaker.setTimeStampField("");
      }
      
      if (selected.equals("<None>")) {
        lagMaker.setAdjustForTrends(false);
      } else {
        lagMaker.setAdjustForTrends(true);
      }
      
      if (m_periodicityCombo.getSelectedItem().
          toString().equals("<Detect automatically>") && 
          selected.equals("<Use an artificial time index>")) {
        
        /*JOptionPane.showConfirmDialog(this, "Cannot automatically detect periodicity" +
        		" when using an artificial time index.", "Forecasting", 
        		JOptionPane.OK_OPTION); */
        throw new Exception("Cannot automatically detect periodicity when using " +
        		"an artificial time index.");
      }
      
      // reset any date-derived periodics to false
      lagMaker.setAddAMIndicator(false); lagMaker.setAddDayOfWeek(false);
      lagMaker.setAddMonthOfYear(false); lagMaker.setAddQuarterOfYear(false);
      lagMaker.setAddWeekendIndicator(false);
      if (!selected.equals("<None>")) {
        checkPeriodicity(forecaster);
      }
      
      if (m_computeConfidence.isSelected()) {
        forecaster.setCalculateConfLimitsForForecasts(getHorizon());
        double confLevel = 
          ((SpinnerNumberModel)m_confidenceLevelSpinner.getModel()).
            getNumber().doubleValue();
        forecaster.setConfidenceLevel(confLevel / 100.0);
      } else {
        forecaster.setCalculateConfLimitsForForecasts(0);
      }
      
      int[] selectedTargets = m_targetPanel.getSelectedAttributes();
      StringBuffer targetBuf = new StringBuffer();
      for (int i = 0; i < selectedTargets.length; i++) {
        targetBuf.
          append(m_targetHeader.attribute(selectedTargets[i]).name()).append(",");
      }
      String temp = targetBuf.substring(0, targetBuf.lastIndexOf(","));
      if (temp.length() == 0) {
        throw new Exception("You must select some fields to forecast!");
      }
      forecaster.setFieldsToForecast(temp);
    }
  }
  
  public void applyToEvaluation(TSEvaluation eval, 
      WekaForecaster forecaster) throws Exception {
    int horizon = 
      ((SpinnerNumberModel) m_horizonSpinner.getModel()).getNumber().intValue();
    if (horizon < 1) {
      throw new Exception("Must specify a non-zero number of steps to" +
      		"forecast into the future");
    }
    
    eval.setHorizon(horizon);
    eval.setPrimeWindowSize(forecaster.getTSLagMaker().getMaxLag());
  }

  /**
   * Tests the simple config panel from the command line.
   *
   * @param args must contain the name of an arff file to load.
   */
  public static void main(String[] args) {

    try {
      if (args.length == 0) {
        throw new Exception("supply the name of an arff file");
      }
      Instances i = new Instances(new java.io.BufferedReader(
                                  new java.io.FileReader(args[0])));
      SimpleConfigPanel scp = new SimpleConfigPanel(null);
      scp.setInstances(i);
      final javax.swing.JFrame jf =
        new javax.swing.JFrame("Simple Config Panel");
      jf.getContentPane().setLayout(new BorderLayout());
      jf.getContentPane().add(scp, BorderLayout.CENTER);
      jf.addWindowListener(new java.awt.event.WindowAdapter() {
        public void windowClosing(java.awt.event.WindowEvent e) {
          jf.dispose();
          System.exit(0);
        }
      });
      jf.pack();
      jf.setVisible(true);
    } catch (Exception ex) {
      ex.printStackTrace();
      System.err.println(ex.getMessage());
    }
  }
}
